@@ -118,7 +118,7 @@ int
 HPCC_TestFFT(HPCC_Params *params, int doIO, double *UGflops, int *Un, int *Ufailure) {
   int rv, n, failure = 1;
   double Gflops;
-  FILE *outFile;
+  FILE *outFile = NULL;
 
   if (doIO) {
     outFile = fopen( params->outFname, "a" );
@@ -127,6 +127,11 @@ HPCC_TestFFT(HPCC_Params *params, int doIO, double *UGflops, int *Un, int *Ufail
       fprintf( outFile, "Cannot open output file.\n" );
       return 1;
     }
+  } else {
+    outFile = fopen( "/dev/null", "w" ); /* special filename Unix file systems */
+    if (! outFile) {
+      outFile = fopen( "nul", "w"); /* special filename on Windows, produces no output */
+    }
   }
 
   n = 0;
@@ -135,9 +140,11 @@ HPCC_TestFFT(HPCC_Params *params, int doIO, double *UGflops, int *Un, int *Ufail
 
   if (doIO) {
     fflush( outFile );
-    fclose( outFile );
   }
 
+  if (outFile)
+    fclose( outFile );
+
   if (UGflops) *UGflops = Gflops;
   if (Un) *Un = n;
   if (Ufailure) *Ufailure = failure;
