@@ -115,7 +115,6 @@
 #include "RandomAccess.h"
 #include "buckets.h"
 #include "time_bound.h"
-#include "verification.h"
 
 #define CHUNK       MAX_TOTAL_PENDING_UPDATES
 #define CHUNKBIG    (32*CHUNK)
@@ -125,25 +124,11 @@
 
 #ifdef RA_SANDIA_OPT2
 void
-HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
-                              u64Int TableSize,
-                              s64Int LocalTableSize,
-                              u64Int MinLocalTableSize,
-                              u64Int GlobalStartMyProc,
-                              u64Int Top,
-                              int logNumProcs,
-                              int NumProcs,
-                              int Remainder,
-                              int MyProc,
-                              s64Int ProcNumUpdates,
-                              MPI_Datatype INT64_DT,
-                              MPI_Status *finish_statuses,
-                              MPI_Request *finish_req)
-{
-  int i,j;
-  int ipartner,iterate,niterate,npartition,proclo,nlower,nupper,procmid;
-  int ndata,nkeep,nsend,nrecv,index, nfrac;
-  u64Int ran,datum,nglobalm1,indexmid;
+HPCC_AnyNodesMPIRandomAccessUpdate_LCG(HPCC_RandomAccess_tabparams_t tparams) {
+  int i, ipartner,npartition,proclo,nlower,nupper,procmid;
+  int ndata,nkeep,nsend,nrecv,nfrac;
+  s64Int iterate, niterate;
+  u64Int ran,datum,nglobalm1,indexmid, index;
   u64Int *data,*send, *offsets;
   MPI_Status status;
 
@@ -155,15 +140,15 @@ HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
   data = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
   send = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
 
-  ran = HPCC_starts_LCG(4*GlobalStartMyProc);
+  ran = HPCC_starts_LCG(4*tparams.GlobalStartMyProc);
 
-  offsets = (u64Int *) malloc((NumProcs+1)*sizeof(u64Int));
-  MPI_Allgather(&GlobalStartMyProc,1,INT64_DT,offsets,1,INT64_DT,
+  offsets = (u64Int *) malloc((tparams.NumProcs+1)*sizeof(u64Int));
+  MPI_Allgather(&tparams.GlobalStartMyProc,1,tparams.dtype64,offsets,1,tparams.dtype64,
                 MPI_COMM_WORLD);
-  offsets[NumProcs] = TableSize;
+  offsets[tparams.NumProcs] = tparams.TableSize;
 
-  niterate = 4 * TableSize / NumProcs / CHUNK + 1;
-  nglobalm1 = 64 - logTableSize;
+  niterate = 4 * tparams.TableSize / tparams.NumProcs / CHUNK + 1;
+  nglobalm1 = 64 - tparams.logTableSize;
 
   /* actual update loop: this is only section that should be timed */
 
@@ -174,7 +159,7 @@ HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
     }
     ndata = CHUNK;
 
-    npartition = NumProcs;
+    npartition = tparams.NumProcs;
     proclo = 0;
     while (npartition > 1) {
       nlower = npartition/2;
@@ -183,7 +168,7 @@ HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
       indexmid = offsets[procmid];
 
       nkeep = nsend = 0;
-      if (MyProc < procmid) {
+      if (tparams.MyProc < procmid) {
         for (i = 0; i < ndata; i++) {
           if ((data[i] >> nglobalm1) >= indexmid) send[nsend++] = data[i];
           else data[nkeep++] = data[i];
@@ -196,48 +181,48 @@ HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
       }
 
       if (nlower == nupper) {
-        if (MyProc < procmid) ipartner = MyProc + nlower;
-        else ipartner = MyProc - nlower;
-        MPI_Sendrecv(send,nsend,INT64_DT,ipartner,0,&data[nkeep],
-                     CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-        MPI_Get_count(&status,INT64_DT,&nrecv);
+        if (tparams.MyProc < procmid) ipartner = tparams.MyProc + nlower;
+        else ipartner = tparams.MyProc - nlower;
+        MPI_Sendrecv(send,nsend,tparams.dtype64,ipartner,0,&data[nkeep],
+                     CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+        MPI_Get_count(&status,tparams.dtype64,&nrecv);
         ndata = nkeep + nrecv;
       } else {
-        if (MyProc < procmid) {
-          nfrac = (nlower - (MyProc-proclo)) * nsend / nupper;
-          ipartner = MyProc + nlower;
-          MPI_Sendrecv(send,nfrac,INT64_DT,ipartner,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+        if (tparams.MyProc < procmid) {
+          nfrac = (nlower - (tparams.MyProc-proclo)) * nsend / nupper;
+          ipartner = tparams.MyProc + nlower;
+          MPI_Sendrecv(send,nfrac,tparams.dtype64,ipartner,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           nkeep += nrecv;
-          MPI_Sendrecv(&send[nfrac],nsend-nfrac,INT64_DT,ipartner+1,0,
-                       &data[nkeep],CHUNKBIG,INT64_DT,
+          MPI_Sendrecv(&send[nfrac],nsend-nfrac,tparams.dtype64,ipartner+1,0,
+                       &data[nkeep],CHUNKBIG,tparams.dtype64,
                        ipartner+1,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
-        } else if (MyProc > procmid && MyProc < procmid+nlower) {
-          nfrac = (MyProc - procmid) * nsend / nlower;
-          ipartner = MyProc - nlower;
-          MPI_Sendrecv(&send[nfrac],nsend-nfrac,INT64_DT,ipartner,0,
-                       &data[nkeep],CHUNKBIG,INT64_DT,
+        } else if (tparams.MyProc > procmid && tparams.MyProc < procmid+nlower) {
+          nfrac = (tparams.MyProc - procmid) * nsend / nlower;
+          ipartner = tparams.MyProc - nlower;
+          MPI_Sendrecv(&send[nfrac],nsend-nfrac,tparams.dtype64,ipartner,0,
+                       &data[nkeep],CHUNKBIG,tparams.dtype64,
                        ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           nkeep += nrecv;
-          MPI_Sendrecv(send,nfrac,INT64_DT,ipartner-1,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner-1,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Sendrecv(send,nfrac,tparams.dtype64,ipartner-1,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner-1,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
         } else {
-          if (MyProc == procmid) ipartner = MyProc - nlower;
-          else ipartner = MyProc - nupper;
-          MPI_Sendrecv(send,nsend,INT64_DT,ipartner,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          if (tparams.MyProc == procmid) ipartner = tparams.MyProc - nlower;
+          else ipartner = tparams.MyProc - nupper;
+          MPI_Sendrecv(send,nsend,tparams.dtype64,ipartner,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
         }
       }
 
-      if (MyProc < procmid) npartition = nlower;
+      if (tparams.MyProc < procmid) npartition = nlower;
       else {
         proclo = procmid;
         npartition = nupper;
@@ -246,7 +231,7 @@ HPCC_AnyNodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
 
     for (i = 0; i < ndata; i++) {
       datum = data[i];
-      index = (datum >> nglobalm1) - GlobalStartMyProc;
+      index = (datum >> nglobalm1) - tparams.GlobalStartMyProc;
       HPCC_Table[index] ^= datum;
     }
   }
@@ -310,18 +295,17 @@ void sort_data(u64Int *source, u64Int *nomatch, u64Int *match, int number,
 /* Manual unrolling is a significant win if -Msafeptr is used -KDU */
 
 static
-void update_table(u64Int *data, u64Int *table, int number, int nglobalm1, int nlocalm1)
-{
-  int i,dindex,index;
+void update_table(u64Int *data, u64Int *table, int number, int nglobalm1, u64Int nlocalm1) {
+  int i,dindex;
   int div_num = number / 8;
   int loop_total = div_num * 8;
-  u64Int index0,index1,index2,index3,index4,index5,index6,index7;
+  u64Int index,index0,index1,index2,index3,index4,index5,index6,index7;
   u64Int ltable0,ltable1,ltable2,ltable3,ltable4,ltable5,ltable6,ltable7;
 
   for (i = 0; i < div_num; i++) {
     dindex = i*8;
 
-    index0 = (data[dindex] >> nglobalm1) & nlocalm1;
+    index0 = (data[dindex  ] >> nglobalm1) & nlocalm1;
     index1 = (data[dindex+1] >> nglobalm1) & nlocalm1;
     index2 = (data[dindex+2] >> nglobalm1) & nlocalm1;
     index3 = (data[dindex+3] >> nglobalm1) & nlocalm1;
@@ -356,25 +340,11 @@ void update_table(u64Int *data, u64Int *table, int number, int nglobalm1, int nl
 }
 
 void
-HPCC_Power2NodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
-                                 u64Int TableSize,
-                                 s64Int LocalTableSize,
-                                 u64Int MinLocalTableSize,
-                                 u64Int GlobalStartMyProc,
-                                 u64Int Top,
-                                 int logNumProcs,
-                                 int NumProcs,
-                                 int Remainder,
-                                 int MyProc,
-                                 s64Int ProcNumUpdates,
-                                 MPI_Datatype INT64_DT,
-                                 MPI_Status *finish_statuses,
-                                 MPI_Request *finish_req)
-{
-  int i,j,k;
-  int logTableLocal,ipartner,iterate,niterate,iter_mod;
-  int ndata,nkeep,nsend,nrecv, nglobalm1, nlocalm1, nkept;
-  u64Int ran,datum;
+HPCC_Power2NodesMPIRandomAccessUpdate_LCG(HPCC_RandomAccess_tabparams_t tparams) {
+  int i, j, k, logTableLocal, ipartner;
+  int ndata, nkeep, nsend, nrecv, nglobalm1, nkept;
+  s64Int iterate, niterate, iter_mod;
+  u64Int ran, nlocalm1;
   u64Int *data,*send,*send1,*send2;
   u64Int *recv[PITER][MAXLOGPROCS];
   MPI_Status status;
@@ -389,15 +359,15 @@ HPCC_Power2NodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
   send = send1;
 
   for (j = 0; j < PITER; j++)
-    for (i = 0; i < logNumProcs; i++)
+    for (i = 0; i < tparams.logNumProcs; i++)
       recv[j][i] = (u64Int *) malloc(sizeof(u64Int)*RCHUNK);
 
-  ran = HPCC_starts_LCG(4*GlobalStartMyProc);
+  ran = HPCC_starts_LCG(4*tparams.GlobalStartMyProc);
 
-  niterate = ProcNumUpdates / CHUNK;
-  logTableLocal = logTableSize - logNumProcs;
-  nglobalm1 = 64 - logTableSize;
-  nlocalm1 = LocalTableSize - 1;
+  niterate = tparams.ProcNumUpdates / CHUNK;
+  logTableLocal = tparams.logTableSize - tparams.logNumProcs;
+  nglobalm1 = 64 - tparams.logTableSize;
+  nlocalm1 = (u64Int)(tparams.LocalTableSize - 1);
 
   /* actual update loop: this is only section that should be timed */
 
@@ -412,21 +382,21 @@ HPCC_Power2NodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
 
     if (iter_mod == 0)
       for (k = 0; k < PITER; k++)
-        for (j = 0; j < logNumProcs; j++) {
-          ipartner = (1 << j) ^ MyProc;
-          MPI_Irecv(recv[k][j],RCHUNK,INT64_DT,ipartner,0,MPI_COMM_WORLD,
+        for (j = 0; j < tparams.logNumProcs; j++) {
+          ipartner = (1 << j) ^ tparams.MyProc;
+          MPI_Irecv(recv[k][j],RCHUNK,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,
                     &request[k][j]);
         }
 
-    for (j = 0; j < logNumProcs; j++) {
+    for (j = 0; j < tparams.logNumProcs; j++) {
       nkeep = nsend = 0;
       send = (send == send1) ? send2 : send1;
-      ipartner = (1 << j) ^ MyProc;
-      if (ipartner > MyProc) {
+      ipartner = (1 << j) ^ tparams.MyProc;
+      if (ipartner > tparams.MyProc) {
       	sort_data(data,data,send,nkept,&nkeep,&nsend,nglobalm1 + logTableLocal+j);
         if (j > 0) {
           MPI_Wait(&request[iter_mod][j-1],&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
       	  sort_data(recv[iter_mod][j-1],data,send,nrecv,&nkeep,
                     &nsend,nglobalm1 + logTableLocal+j);
         }
@@ -434,21 +404,21 @@ HPCC_Power2NodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
         sort_data(data,send,data,nkept,&nsend,&nkeep,nglobalm1 + logTableLocal+j);
         if (j > 0) {
           MPI_Wait(&request[iter_mod][j-1],&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           sort_data(recv[iter_mod][j-1],send,data,nrecv,&nsend,
                     &nkeep,nglobalm1 + logTableLocal+j);
         }
       }
       if (j > 0) MPI_Wait(&srequest,&status);
-      MPI_Isend(send,nsend,INT64_DT,ipartner,0,MPI_COMM_WORLD,&srequest);
-      if (j == (logNumProcs - 1)) update_table(data,HPCC_Table,nkeep,nglobalm1, nlocalm1);
+      MPI_Isend(send,nsend,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&srequest);
+      if (j == (tparams.logNumProcs - 1)) update_table(data,HPCC_Table,nkeep,nglobalm1, nlocalm1);
       nkept = nkeep;
     }
 
-    if (logNumProcs == 0) update_table(data,HPCC_Table,nkept,nglobalm1, nlocalm1);
+    if (tparams.logNumProcs == 0) update_table(data,HPCC_Table,nkept,nglobalm1, nlocalm1);
     else {
       MPI_Wait(&request[iter_mod][j-1],&status);
-      MPI_Get_count(&status,INT64_DT,&nrecv);
+      MPI_Get_count(&status,tparams.dtype64,&nrecv);
       update_table(recv[iter_mod][j-1],HPCC_Table,nrecv,nglobalm1, nlocalm1);
       MPI_Wait(&srequest,&status);
     }
@@ -459,7 +429,7 @@ HPCC_Power2NodesMPIRandomAccessUpdate_LCG(u64Int logTableSize,
   /* clean up: should not really be part of this timed routine */
 
   for (j = 0; j < PITER; j++)
-    for (i = 0; i < logNumProcs; i++) free(recv[j][i]);
+    for (i = 0; i < tparams.logNumProcs; i++) free(recv[j][i]);
 
   free(data);
   free(send1);
