@@ -115,32 +115,17 @@
 #include "RandomAccess.h"
 #include "buckets.h"
 #include "time_bound.h"
-#include "verification.h"
 
 #define CHUNK    MAX_TOTAL_PENDING_UPDATES
 #define CHUNKBIG (32*CHUNK)
 
 #ifdef RA_SANDIA_NOPT
 void
-AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
-                              u64Int TableSize,
-                              s64Int LocalTableSize,
-                              u64Int MinLocalTableSize,
-                              u64Int GlobalStartMyProc,
-                              u64Int Top,
-                              int logNumProcs,
-                              int NumProcs,
-                              int Remainder,
-                              int MyProc,
-                              s64Int ProcNumUpdates,
-                              MPI_Datatype INT64_DT,
-                              MPI_Status *finish_statuses,
-                              MPI_Request *finish_req)
-{
-  int i,j;
-  int ipartner,iterate,niterate,npartition,proclo,nlower,nupper,procmid;
-  int ndata,nkeep,nsend,nrecv,index, nfrac;
-  u64Int ran,datum,nglobalm1,indexmid;
+AnyNodesMPIRandomAccessUpdate(HPCC_RandomAccess_tabparams_t tparams) {
+  int i, ipartner,npartition,proclo,nlower,nupper,procmid;
+  int ndata,nkeep,nsend,nrecv,nfrac;
+  s64Int iterate, niterate;
+  u64Int ran,datum,nglobalm1,indexmid, index;
   u64Int *data,*send, *offsets;
   MPI_Status status;
 
@@ -152,15 +137,15 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
   data = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
   send = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
 
-  ran = HPCC_starts(4*GlobalStartMyProc);
+  ran = HPCC_starts(4*tparams.GlobalStartMyProc);
 
-  offsets = (u64Int *) malloc((NumProcs+1)*sizeof(u64Int));
-  MPI_Allgather(&GlobalStartMyProc,1,INT64_DT,offsets,1,INT64_DT,
+  offsets = (u64Int *) malloc((tparams.NumProcs+1)*sizeof(u64Int));
+  MPI_Allgather(&tparams.GlobalStartMyProc,1,tparams.dtype64,offsets,1,tparams.dtype64,
                 MPI_COMM_WORLD);
-  offsets[NumProcs] = TableSize;
+  offsets[tparams.NumProcs] = tparams.TableSize;
 
-  niterate = 4 * TableSize / NumProcs / CHUNK + 1;
-  nglobalm1 = TableSize - 1;
+  niterate = 4 * tparams.TableSize / tparams.NumProcs / CHUNK + 1;
+  nglobalm1 = tparams.TableSize - 1;
 
   /* actual update loop: this is only section that should be timed */
 
@@ -171,7 +156,7 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
     }
     ndata = CHUNK;
 
-    npartition = NumProcs;
+    npartition = tparams.NumProcs;
     proclo = 0;
     while (npartition > 1) {
       nlower = npartition/2;
@@ -180,7 +165,7 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
       indexmid = offsets[procmid];
 
       nkeep = nsend = 0;
-      if (MyProc < procmid) {
+      if (tparams.MyProc < procmid) {
         for (i = 0; i < ndata; i++) {
           if ((data[i] & nglobalm1) >= indexmid) send[nsend++] = data[i];
           else data[nkeep++] = data[i];
@@ -193,48 +178,48 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
       }
 
       if (nlower == nupper) {
-        if (MyProc < procmid) ipartner = MyProc + nlower;
-        else ipartner = MyProc - nlower;
-        MPI_Sendrecv(send,nsend,INT64_DT,ipartner,0,&data[nkeep],
-                     CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-        MPI_Get_count(&status,INT64_DT,&nrecv);
+        if (tparams.MyProc < procmid) ipartner = tparams.MyProc + nlower;
+        else ipartner = tparams.MyProc - nlower;
+        MPI_Sendrecv(send,nsend,tparams.dtype64,ipartner,0,&data[nkeep],
+                     CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+        MPI_Get_count(&status,tparams.dtype64,&nrecv);
         ndata = nkeep + nrecv;
       } else {
-        if (MyProc < procmid) {
-          nfrac = (nlower - (MyProc-proclo)) * nsend / nupper;
-          ipartner = MyProc + nlower;
-          MPI_Sendrecv(send,nfrac,INT64_DT,ipartner,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+        if (tparams.MyProc < procmid) {
+          nfrac = (nlower - (tparams.MyProc-proclo)) * nsend / nupper;
+          ipartner = tparams.MyProc + nlower;
+          MPI_Sendrecv(send,nfrac,tparams.dtype64,ipartner,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           nkeep += nrecv;
-          MPI_Sendrecv(&send[nfrac],nsend-nfrac,INT64_DT,ipartner+1,0,
-                       &data[nkeep],CHUNKBIG,INT64_DT,
+          MPI_Sendrecv(&send[nfrac],nsend-nfrac,tparams.dtype64,ipartner+1,0,
+                       &data[nkeep],CHUNKBIG,tparams.dtype64,
                        ipartner+1,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
-        } else if (MyProc > procmid && MyProc < procmid+nlower) {
-          nfrac = (MyProc - procmid) * nsend / nlower;
-          ipartner = MyProc - nlower;
-          MPI_Sendrecv(&send[nfrac],nsend-nfrac,INT64_DT,ipartner,0,
-                       &data[nkeep],CHUNKBIG,INT64_DT,
+        } else if (tparams.MyProc > procmid && tparams.MyProc < procmid+nlower) {
+          nfrac = (tparams.MyProc - procmid) * nsend / nlower;
+          ipartner = tparams.MyProc - nlower;
+          MPI_Sendrecv(&send[nfrac],nsend-nfrac,tparams.dtype64,ipartner,0,
+                       &data[nkeep],CHUNKBIG,tparams.dtype64,
                        ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           nkeep += nrecv;
-          MPI_Sendrecv(send,nfrac,INT64_DT,ipartner-1,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner-1,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          MPI_Sendrecv(send,nfrac,tparams.dtype64,ipartner-1,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner-1,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
         } else {
-          if (MyProc == procmid) ipartner = MyProc - nlower;
-          else ipartner = MyProc - nupper;
-          MPI_Sendrecv(send,nsend,INT64_DT,ipartner,0,&data[nkeep],
-                       CHUNKBIG,INT64_DT,ipartner,0,MPI_COMM_WORLD,&status);
-          MPI_Get_count(&status,INT64_DT,&nrecv);
+          if (tparams.MyProc == procmid) ipartner = tparams.MyProc - nlower;
+          else ipartner = tparams.MyProc - nupper;
+          MPI_Sendrecv(send,nsend,tparams.dtype64,ipartner,0,&data[nkeep],
+                       CHUNKBIG,tparams.dtype64,ipartner,0,MPI_COMM_WORLD,&status);
+          MPI_Get_count(&status,tparams.dtype64,&nrecv);
           ndata = nkeep + nrecv;
         }
       }
 
-      if (MyProc < procmid) npartition = nlower;
+      if (tparams.MyProc < procmid) npartition = nlower;
       else {
         proclo = procmid;
         npartition = nupper;
@@ -243,7 +228,7 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
 
     for (i = 0; i < ndata; i++) {
       datum = data[i];
-      index = (datum & nglobalm1) - GlobalStartMyProc;
+      index = (datum & nglobalm1) - tparams.GlobalStartMyProc;
       HPCC_Table[index] ^= datum;
     }
   }
@@ -256,25 +241,11 @@ AnyNodesMPIRandomAccessUpdate(u64Int logTableSize,
 }
 
 void
-Power2NodesMPIRandomAccessUpdate(u64Int logTableSize,
-                                 u64Int TableSize,
-                                 s64Int LocalTableSize,
-                                 u64Int MinLocalTableSize,
-                                 u64Int GlobalStartMyProc,
-                                 u64Int Top,
-                                 int logNumProcs,
-                                 int NumProcs,
-                                 int Remainder,
-                                 int MyProc,
-                                 s64Int ProcNumUpdates,
-                                 MPI_Datatype INT64_DT,
-                                 MPI_Status *finish_statuses,
-                                 MPI_Request *finish_req)
-{
-  int i,j;
-  int logTableLocal,ipartner,iterate,niterate;
-  int ndata,nkeep,nsend,nrecv,index,nlocalm1;
-  u64Int ran,datum,procmask;
+Power2NodesMPIRandomAccessUpdate(HPCC_RandomAccess_tabparams_t tparams) {
+  int i, j, logTableLocal,ipartner;
+  int ndata, nkeep, nsend, nrecv;
+  s64Int iterate, niterate;
+  u64Int ran,datum,procmask, nlocalm1, index;
   u64Int *data,*send;
   MPI_Status status;
 
@@ -283,11 +254,11 @@ Power2NodesMPIRandomAccessUpdate(u64Int logTableSize,
   data = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
   send = (u64Int *) malloc(CHUNKBIG*sizeof(u64Int));
 
-  ran = HPCC_starts(4*GlobalStartMyProc);
+  ran = HPCC_starts(4*tparams.GlobalStartMyProc);
 
-  niterate = ProcNumUpdates / CHUNK;
-  logTableLocal = logTableSize - logNumProcs;
-  nlocalm1 = LocalTableSize - 1;
+  niterate = tparams.ProcNumUpdates / CHUNK;
+  logTableLocal = tparams.logTableSize - tparams.logNumProcs;
+  nlocalm1 = (u64Int)(tparams.LocalTableSize - 1);
 
   /* actual update loop: this is only section that should be timed */
 
@@ -298,11 +269,11 @@ Power2NodesMPIRandomAccessUpdate(u64Int logTableSize,
     }
     ndata = CHUNK;
 
-    for (j = 0; j < logNumProcs; j++) {
+    for (j = 0; j < tparams.logNumProcs; j++) {
       nkeep = nsend = 0;
-      ipartner = (1 << j) ^ MyProc;
+      ipartner = (1 << j) ^ tparams.MyProc;
       procmask = ((u64Int) 1) << (logTableLocal + j);
-      if (ipartner > MyProc) {
+      if (ipartner > tparams.MyProc) {
         for (i = 0; i < ndata; i++) {
           if (data[i] & procmask) send[nsend++] = data[i];
           else data[nkeep++] = data[i];
@@ -314,10 +285,10 @@ Power2NodesMPIRandomAccessUpdate(u64Int logTableSize,
         }
       }
 
-      MPI_Sendrecv(send,nsend,INT64_DT,ipartner,0,
-                   &data[nkeep],CHUNKBIG,INT64_DT,
+      MPI_Sendrecv(send,nsend,tparams.dtype64,ipartner,0,
+                   &data[nkeep],CHUNKBIG,tparams.dtype64,
                    ipartner,0,MPI_COMM_WORLD,&status);
-      MPI_Get_count(&status,INT64_DT,&nrecv);
+      MPI_Get_count(&status,tparams.dtype64,&nrecv);
       ndata = nkeep + nrecv;
     }
 
